@@ -6,8 +6,12 @@
 // This file may be distributed under the terms of the GNU LGPLv3 license.
 
 #include "biosvar.h" // SET_BDA
-#include "util.h" // debug_enter
 #include "bregs.h" // struct bregs
+#include "hw/serialio.h" // SEROFF_IER
+#include "output.h" // debug_enter
+#include "romfile.h" // romfile_loadint
+#include "stacks.h" // yield
+#include "util.h" // serial_setup
 
 
 /****************************************************************
@@ -17,6 +21,9 @@
 static u16
 detect_serial(u16 port, u8 timeout, u8 count)
 {
+    if (CONFIG_DEBUG_SERIAL && port == CONFIG_DEBUG_SERIAL_PORT
+        && !romfile_loadint("etc/advertise-serial-debug-port", 1))
+        return 0;
     outb(0x02, port+SEROFF_IER);
     u8 ier = inb(port+SEROFF_IER);
     if (ier != 0x02)
@@ -91,7 +98,7 @@ handle_1401(struct bregs *regs)
     u16 addr = getComAddr(regs);
     if (!addr)
         return;
-    u32 end = calc_future_timer_ticks(GET_BDA(com_timeout[regs->dx]));
+    u32 end = irqtimer_calc_ticks(GET_BDA(com_timeout[regs->dx]));
     for (;;) {
         u8 lsr = inb(addr+SEROFF_LSR);
         if ((lsr & 0x60) == 0x60) {
@@ -101,7 +108,7 @@ handle_1401(struct bregs *regs)
             regs->ah = lsr;
             break;
         }
-        if (check_timer(end)) {
+        if (irqtimer_check(end)) {
             // Timed out - can't write data.
             regs->ah = lsr | 0x80;
             break;
@@ -118,7 +125,7 @@ handle_1402(struct bregs *regs)
     u16 addr = getComAddr(regs);
     if (!addr)
         return;
-    u32 end = calc_future_timer_ticks(GET_BDA(com_timeout[regs->dx]));
+    u32 end = irqtimer_calc_ticks(GET_BDA(com_timeout[regs->dx]));
     for (;;) {
         u8 lsr = inb(addr+SEROFF_LSR);
         if (lsr & 0x01) {
@@ -127,7 +134,7 @@ handle_1402(struct bregs *regs)
             regs->ah = lsr;
             break;
         }
-        if (check_timer(end)) {
+        if (irqtimer_check(end)) {
             // Timed out - can't read data.
             regs->ah = lsr | 0x80;
             break;
@@ -234,7 +241,7 @@ handle_1700(struct bregs *regs)
     if (!addr)
         return;
 
-    u32 end = calc_future_timer_ticks(GET_BDA(lpt_timeout[regs->dx]));
+    u32 end = irqtimer_calc_ticks(GET_BDA(lpt_timeout[regs->dx]));
 
     outb(regs->al, addr);
     u8 val8 = inb(addr+2);
@@ -249,7 +256,7 @@ handle_1700(struct bregs *regs)
             regs->ah = v ^ 0x48;
             break;
         }
-        if (check_timer(end)) {
+        if (irqtimer_check(end)) {
             // Timeout
             regs->ah = (v ^ 0x48) | 0x01;
             break;
