@@ -25,6 +25,6 @@ void iomemcpy(void *d, const void *s, u32 len);
 void *memmove(void *d, const void *s, size_t len);
 char *strtcpy(char *dest, const char *src, size_t len);
 char *strchr(const char *s, int c);
-void nullTrailingSpace(char *buf);
+char *nullTrailingSpace(char *buf);
 
 #endif // string.h
